@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003, 2004, 2005, 2006 Robert Lougher <rob@lougher.demon.co.uk>.
+ * Copyright (C) 2003, 2004, 2005, 2006 Robert Lougher <rob@lougher.org.uk>.
  *
  * This file is part of JamVM.
  *
@@ -275,6 +275,8 @@
 #define CONSTANT_NameAndType            12
 
 #define CONSTANT_Resolved               20
+#define CONSTANT_ResolvedClass          25
+#define CONSTANT_ResolvedString         26
 #define CONSTANT_Locked                 21
 
 #define ACC_PUBLIC              0x0001
@@ -300,8 +302,8 @@
 #define T_DOUBLE                7
 #define T_BYTE                  8
 #define T_SHORT                 9
-#define T_INT                   10
-#define T_LONG                  11
+#define T_INT                  10
+#define T_LONG                 11
 
 /* Class states */
 
@@ -316,14 +318,16 @@
 
 /* Class flags */
 
-#define REFERENCE               1
-#define SOFT_REFERENCE          2
-#define WEAK_REFERENCE          4
-#define PHANTOM_REFERENCE       8
-#define FINALIZED               16 
-#define CLASS_LOADER            32
-#define CLASS_CLASH             64
-#define VMTHROWABLE             128
+#define CLASS_CLASS             1
+#define REFERENCE               2
+#define SOFT_REFERENCE          4
+#define WEAK_REFERENCE          8
+#define PHANTOM_REFERENCE      16
+#define FINALIZED              32 
+#define CLASS_LOADER           64 
+#define CLASS_CLASH           128
+#define VMTHROWABLE           256 
+#define ANONYMOUS             512
 
 typedef unsigned char           u1;
 typedef unsigned short          u2;
@@ -349,14 +353,11 @@ typedef struct line_no_table_entry {
     u2 line_no;
 } LineNoTableEntry;
 
-typedef struct class {
-   uintptr_t lock;
-   struct class *class;
-} Class;
+typedef struct object Class;
 
 typedef struct object {
    uintptr_t lock;
-   struct class *class;
+   Class *class;
 } Object;
 
 #ifdef DIRECT
@@ -416,6 +417,7 @@ typedef struct methodblock {
    Class *class;
    char *name;
    char *type;
+   char *signature;
    u2 access_flags;
    u2 max_stack;
    u2 max_locals;
@@ -437,6 +439,7 @@ typedef struct fieldblock {
    Class *class;
    char *name;
    char *type;
+   char *signature;
    u2 access_flags;
    u2 constant;
    uintptr_t static_value;
@@ -458,11 +461,12 @@ typedef struct refs_offsets_entry {
 typedef struct classblock {
    uintptr_t pad[CLASS_PAD_SIZE];
    char *name;
+   char *signature;
    char *super_name;
    char *source_file_name;
    Class *super;
    u1 state;
-   u1 flags;
+   u2 flags;
    u2 access_flags;
    u2 interfaces_count;
    u2 fields_count;
@@ -487,21 +491,23 @@ typedef struct classblock {
    u2 *inner_classes;
    int refs_offsets_size;
    RefsOffsetsEntry *refs_offsets_table;
+   u2 enclosing_class;
+   u2 enclosing_method;
 } ClassBlock;
 
 typedef struct frame {
-   MethodBlock *mb;
    CodePntr last_pc;
    uintptr_t *lvars;
    uintptr_t *ostack;
+   MethodBlock *mb;
    struct frame *prev;
 } Frame;
 
 typedef struct jni_frame {
-   MethodBlock *mb;
    Object **next_ref;
    Object **lrefs;
    uintptr_t *ostack;
+   MethodBlock *mb;
    struct frame *prev;
 } JNIFrame;
 
@@ -526,7 +532,7 @@ typedef struct prop {
 #define ARRAY_DATA(arrayRef)            ((void*)(((u4*)(arrayRef+1))+1))
 #define ARRAY_LEN(arrayRef)             *(u4*)(arrayRef+1)
 
-#define IS_CLASS(object)                (!object->class || (object->class == java_lang_Class))
+#define IS_CLASS(object)                (object->class && IS_CLASS_CLASS(CLASS_CB(object->class)))
 
 #define IS_INTERFACE(cb)                (cb->access_flags & ACC_INTERFACE)
 #define IS_SYNTHETIC(cb)                (cb->access_flags & ACC_SYNTHETIC)
@@ -542,10 +548,14 @@ typedef struct prop {
 #define IS_PHANTOM_REFERENCE(cb)	(cb->flags & PHANTOM_REFERENCE)
 #define IS_CLASS_LOADER(cb)		(cb->flags & CLASS_LOADER)
 #define IS_CLASS_DUP(cb)		(cb->flags & CLASS_CLASH)
+#define IS_CLASS_CLASS(cb)		(cb->flags & CLASS_CLASS)
 #define IS_VMTHROWABLE(cb)		(cb->flags & VMTHROWABLE)
-
+#define IS_ANONYMOUS(cb)		(cb->flags & ANONYMOUS)
 #define IS_SPECIAL(cb)			(cb->flags & (REFERENCE | CLASS_LOADER))
 
+#define IS_MEMBER(cb)			cb->declaring_class
+#define IS_LOCAL(cb)			(cb->enclosing_method && !IS_ANONYMOUS(cb))
+
 /* Macros for accessing constant pool entries */
 
 #define CP_TYPE(cp,i)                   cp->type[i]
@@ -593,12 +603,14 @@ typedef struct prop {
    a StackOverflow exception */
 #define STACK_RED_ZONE_SIZE 1*KB
 
+#define JAVA_COMPAT_VERSION "1.4.2"
+
 /* --------------------- Function prototypes  --------------------------- */
 
 /* Alloc */
 
 extern void initialiseAlloc(unsigned long min, unsigned long max, int verbose);
-extern void initialiseGC(int noasyncgc);
+extern void initialiseGC(int noasyncgc, int compact_override, int compact_value);
 extern Class *allocClass();
 extern Object *allocObject(Class *class);
 extern Object *allocTypeArray(int type, int size);
@@ -606,7 +618,9 @@ extern Object *allocArray(Class *class, int size, int el_size);
 extern Object *allocMultiArray(Class *array_class, int dim, intptr_t *count);
 extern Object *cloneObject(Object *ob);
 extern void markRoot(Object *ob);
+extern void markConservativeRoot(Object *ob);
 extern void markObject(Object *ob, int mark, int mark_soft_refs);
+extern uintptr_t getObjectHashcode(Object *ob);
 
 extern void gc1();
 extern void runFinalizers();
@@ -618,6 +632,15 @@ extern unsigned long maxHeapMem();
 extern void *sysMalloc(int n);
 extern void *sysRealloc(void *ptr, int n);
 
+extern void registerStaticObjectRef(Object **ob);
+
+#define registerStaticClassRef(ref) \
+    registerStaticObjectRef(ref);
+
+/* GC support */
+extern void threadReference(Object **ref);
+extern int isMarked(Object *ob);
+
 /* Class */
 
 extern Class *java_lang_Class;
@@ -651,6 +674,12 @@ extern void freeClassLoaderData(Object *class_loader);
 
 extern char *getClassPath();
 extern char *getBootClassPath();
+
+extern void markBootClasses();
+extern void markLoaderClasses(Object *loader, int mark, int mark_soft_refs);
+extern void threadBootClasses();
+extern void threadLoaderClasses(Object *class_loader);
+
 extern void initialiseClass(char *classpath, char *bootpath, char bootpathopt, int verbose);
 
 /* resolve */
@@ -696,6 +725,7 @@ extern CodePntr findCatchBlock(Class *exception);
 extern Object *setStackTrace();
 extern Object *convertStackTrace(Object *vmthrwble);
 extern int mapPC2LineNo(MethodBlock *mb, CodePntr pc_pntr);
+extern void markVMThrowable(Object *vmthrwble, int mark, int mark_soft_refs);
 extern void initialiseException();
 
 #define exceptionOccured0(ee) \
@@ -721,6 +751,8 @@ extern Object *getStringCharsArray(Object *string);
 extern int getStringUtf8Len(Object *string);
 extern char *String2Utf8(Object *string);
 extern char *StringRegion2Utf8(Object *string, int start, int len, char *utf8);
+extern void freeInternedStrings();
+extern void threadInternedStrings();
 extern void initialiseString();
 
 #define Cstr2String(cstr) createString(cstr)
@@ -755,6 +787,8 @@ extern void *nativeLibOpen(char *path);
 extern char *nativeLibMapName(char *name);
 extern void *nativeLibSym(void *handle, char *symbol);
 
+extern int nativeAvailableProcessors();
+
 /* Threading */
 
 extern void initialiseMainThread(int java_stack);
@@ -764,6 +798,7 @@ extern void createJavaThread(Object *jThread, long long stack_size);
 extern void mainThreadSetContextClassLoader(Object *loader);
 extern void mainThreadWaitToExitVM();
 extern void exitVM(int status);
+extern void scanThreads();
 
 /* Monitors */
 
@@ -777,6 +812,9 @@ extern Object *getClassFields(Class *class, int public);
 extern Object *getClassInterfaces(Class *class);
 extern Object *getClassClasses(Class *class, int public);
 extern Class *getDeclaringClass(Class *class);
+extern Class *getEnclosingClass(Class *class);
+extern Object *getEnclosingMethodObject(Class *class);
+extern Object *getEnclosingConstructorObject(Class *class);
 
 extern Object *createWrapperObject(Class *type, uintptr_t *pntr);
 extern uintptr_t *widenPrimitiveValue(int src_idx, int dest_idx, uintptr_t *src, uintptr_t *dest);
@@ -798,6 +836,7 @@ extern Class *getReflectMethodClass();
 
 extern void initialiseJNI();
 extern void *getJNIInterface();
+extern void markJNIGlobalRefs();
 
 /* properties */
 
